# Analyse the lensing results and output tables and figures
#
# (C) 2023 Boud Roukema, Marius Peper, GPL-3+
#
# This Makefile is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation, either version 3 of the License, or (at your
# option) any later version.
#
# This script is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details. See <http://www.gnu.org/licenses/>.

void_matching_dir = $(badir)/void_matching
done_void_matching = $(void_matching_dir)/done_void_matching
done_void_match_plotting = $(void_matching_dir)/done_void_match_plotting
void_matching_tables = mass_def gamma exp sig
void_matching_tables_dat = $(foreach s,$(void_matching_tables),$(void_matching_dir)/void_matches_$(s)_given_2D.dat) \
             $(foreach s,$(void_matching_tables),$(void_matching_dir)/void_matches_$(s)_given_3D.dat)
void_matching_analysis = $(data-publish-dir)/void_match_analysis.dat


.PHONY : analyse-plot-lensing
analyse-plot-lensing: $(done_analyse_plot_file) $(mtexdir)/analyse-plot-lensing.tex

.PHONY : analyse-plot-lensing-tex
analyse-plot-lensing-tex: $(mtexdir)/analyse-plot-lensing.tex

$(void_matching_dir):; mkdir $@

$(void_matching_analysis): $(done_void_matching)

$(done_void_matching): $(done_lensing_file) | $(void_matching_dir) $(data-publish-dir) 
#	Copy the void_match*dat files to $(data-publish-dir) in publishable format.
	for table in $(void_matching_tables); do
	  for nD in 2D 3D; do
	    input_table_dat=$(lensing_dir)/void_matches_$${table}_given_$${nD}.dat
	    output_table_dat=$(data-publish-dir)/void_matches_$${table}_given_$${nD}.dat
	    $(call print-general-metadata, $${output_table_dat}, new)
	    cat $${input_table_dat} >> $${output_table_dat}
	  done
	done
#
#       Call analyse_void_matches.py and place publishable output files in $(data-publish-dir);
#	WARNING: this gives the commit ID of analysing the void matching, not of doing the
#	void matching itself.
	LENSING_DIR=$(lensing_dir) \
	  N_VOID_MATCHES_SIM=$(N_void_matches_sim) \
	  LENSING_SEED_VOID_MATCHES=$(lensing_seed_void_matches) \
	  python $(pythondir)/analyse_void_matches.py > \
	  $(void_matching_dir)/void_match_analysis_raw.dat
	$(call print-general-metadata, $(void_matching_analysis), new)
	grep "^prob_random_uniq:" $(void_matching_dir)/void_match_analysis_raw.dat | \
	   sed -e 's/\\//g' -e 's/vert.*sigma/abs_sigma/' -e 's/vert//' \
	       -e 's/^prob_random_uniq: //' >> $(void_matching_analysis)
#
#
	touch $(done_void_matching)


$(done_void_match_plotting): $(done_void_matching) | $(void_matching_dir)
	INFILE_DIR=$(lensing_dir) \
	  OUTFILE_DIR=$(fig_dir) \
	  python $(pythondir)/plot_voidmatch_accuracy.py
	touch $(done_void_match_plotting)

$(mtexdir)/analyse-plot-lensing.tex: $(done_void_matching) $(done_void_match_plotting)
	printf "%% Automatically produced file.\n\n"
	printf "%% Automatically produced file.\n" > $@
#	Main results
	printf "%snewcommand\\VoidMatchSignificanceTable{\n" "\\" >> $@
	grep "^prob_random_uniq:" $(void_matching_dir)/void_match_analysis_raw.dat | \
	   sed -e 's/^prob_random_uniq: //' | \
	   grep -v "^#" | \
	   sed -e 's/  *\([0-9]\)/   \& \1/g' -e "s/\'/ \\\\\\\\/" \
	       -e "s/e\(-\)0*\([1-9][0-9]*\)/\\\\times 10^{\1\2}/g" >> $@
	printf "}\n" >> $@


	# Name individual values
	for param in Sigma gamma theta sigma; do
	   grep "^prob_random_uniq:.*$${param}" $(void_matching_dir)/void_match_analysis_raw.dat | \
	   sed -e 's/^prob_random_uniq: //' | \
	   grep -v "^#" | \
	   sed -e "s/\([^0-9 ][^0-9 ]*\)  *\([0-9][.0-9e-]*\)  *\([0-9][.0-9e-]*\)  *\([0-9][.0-9e-]*\)  *\([0-9][.0-9e-]*\)/\\\\newcommand\\\\$${param}SignificanceXZthreeDgiventwoD{\2}\n\\\\newcommand\\\\$${param}SignificanceXZtwoDgiventhreeD{\3}\n\\\\newcommand\\\\$${param}SignificanceRthreeDgiventwoD{\4}\n\\\\newcommand\\\\$${param}SignificanceRtwoDgiventhreeD{\5}\n/g" \
	       -e "s/e\(-\)0*\([1-9][0-9]*\)/\\\\times 10^{\1\2}/g" >> $@
	done

	# Median xz T^2 separations
	$(call create_tex_macro, $(void_matching_dir)/void_match_analysis_raw.dat, xz_median_given_threeD_uniq_mass_def, float.1, )
	$(call create_tex_macro, $(void_matching_dir)/void_match_analysis_raw.dat, xz_median_given_threeD_uniq_gamma, float.1, )
	$(call create_tex_macro, $(void_matching_dir)/void_match_analysis_raw.dat, xz_median_given_threeD_uniq_exp, float.1, )
	$(call create_tex_macro, $(void_matching_dir)/void_match_analysis_raw.dat, xz_median_given_threeD_uniq_sig, float.1, )


.PHONY: clean-void-matches

clean-void-matches:
	rm -fv $(done_void_matching) $(void_matching_dir)/void_match*.dat
