## Apply analysis to the voids

## Development mode
ifeq (x$(devmode),x1)
lensing_paramsfile = $(pconfdir)/lensing-params-devmode.conf
else
lensing_paramsfile = $(pconfdir)/lensing-params.conf
endif

#sim_dir = $(BDIR)/analysis/n-body-init/simulation-out # sim_out_dir defined in init-N-body.mk
stack_res_dir = $(BDIR)/analysis/stacking_res
example_lensing_output_file = $(lensing_dir)/mass_deficit_void_cart_gals_0.eps
#example_void_output_file = $(voids_dm_dir)/outputpotcentres/Voids_potC_cat.txt
done_lensing_file = $(lensing_dir)/done-lensing-calculations
#void_elaphro_rad_precision = .1f

py_library_files = $(pythondir)/extra_calculations.py \
	$(pythondir)/read.py \
	$(pythondir)/figures.py \
	$(pythondir)/coordinates.py \
	$(pythondir)/lensing_calculations.py \
	$(pythondir)/lensing_calculations_unit_testable.py
py_analyse_executables = $(pythondir)/lensing.py \
        $(pythondir)/test_local_minima.py \
        $(pythondir)/test_merge_centres.py \
        $(pythondir)/analyse_void_matches.py \
        $(pythondir)/plot_voidmatch_accuracy.py


#ISW_void_cart_gals_0 \
#vel_los_void_cart_gals_0 \
#average_los_velocity_0 \
#theta_vs_kappa_cart_gals_0 \
	average_mass_def \
	theta_deviation_void_cart_gals \
	rads_vs_prob_mass_def \
	rads_vs_prob_expansion \
	rads_vs_prob_sigma \
#sigma_vs_gamma_cart_gals_0
output_eps_files = mass_deficit_void_cart_gals \
	weak_lensing_gamma_grid_gals \
	densities_cart_void_gals \
	potential_cart_void_gals \
	real_theta_void_cart_gals \
	real_sigma_void_cart_gals \
	img_sigma_void_cart_gals \
	mag_sigma_void_cart_gals \
	average_mass_def \
	average_expansion \
	average_weak_lensing_gamma \
	average_sigma \
	average_mass_def_revolver \
	average_expansion_revolver \
	average_weak_lensing_gamma_revolver \
	average_sigma_revolver


fig_dir = $(texbdir)/fig

REVOLVER_PATH = $(ilibpydir)/Revolver
LD_LIBRARY_PATH_PYTHON=${REVOLVER_PATH}/python_tools:${LD_LIBRARY_PATH}


#aliases
.PHONY : lensing
lensing: $(done_lensing_file) $(mtexdir)/lensing.tex

.PHONY : lensing.tex
lensing.tex: $(mtexdir)/lensing.tex

$(done_lensing_file): $(example_lensing_output_file) $(done_create_galaxies_file) $(done_detect_voids_file) $(done_density_file)

$(example_lensing_output_file): \
              $(done_create_galaxies_file) \
              $(done_detect_voids_file) \
              $(done_density_file) \
	      $(void_paramsfile) \
	      $(lensing_paramsfile)
	python --version
	cp -pv $(bashdir)/run-lensing $(ibdir)
	cp -pv $(py_library_files) $(ilibpydir)
	cp -pv $(py_analyse_executables) $(ibdir)
	mkdir -p $(lensing_dir)
#	cp -pv $(ibdir)/optical_scalar_solver.so $(lensing_dir)
#	cp -pv $(ibdir)/optical_scalar_solver.so $(REVOLVER_PATH)
#	cp -pv $(ibdir)/gsl_integration.so $(lensing_dir)
#	cp -pv $(ibdir)/gsl_integration.so $(REVOLVER_PATH)
	cd $(lensing_dir)
	rm -fv densities_voronoi.npy
	ln -s $(density_output_file) .
	mkdir -p $(fig_dir)
	if [ "x$(stacking)" = "xYES" ] ; then
	  if [ ! -d $(stack_res_dir) ]; then
	    mkdir -p $(stack_res_dir)
	  fi
	  if [ ! -f $(stack_res_dir)/current_nstack.txt ]; then
	    echo 1 > $(stack_res_dir)/current_nstack.txt
	  fi
	  rm -f "$(stack_res_dir)/do_stacking"
	  NSTACK=$$(cat $(stack_res_dir)/current_nstack.txt)
	  if [ $${NSTACK} -ge $(N_stack) ] ; then
	    touch $(stack_res_dir)/do_stacking
	  fi
	else
	  NSTACK=1
	  # Create the file to fullfil the dependency later
	  #touch $(stack_res_dir)/do_stacking
	fi
	printf "\nWill call run-lensing front end to lensing.py; devmode=%s .\n" $(devmode)
	LBOX=$(Lbox) HUBBLE0=$(Hubble) TIME_STEP=$(time_step) \
	 SIM_DIR=$(sim_out_dir) \
	 VOID_DM_DIR=$(voids_dm_dir) \
	 VOID_GALS_DIR=$(voids_gals_dir) \
	 GAL_DIR=$(gal_dir) \
	 LENSING_DIR=$(lensing_dir) \
	 NCROOT=$(Ncroot) \
	 OMEGA_M=$(OmegaM) \
	 OMEGA_LAMBDA=$(OmegaL) \
	 MAIN_VOID_REDSHIFT=$(main_void_redshift) \
	 finishing_time_step_margin=$(finishing_time_step_margin) \
	 LD_LIBRARY_PATH=$(LD_LIBRARY_PATH_PYTHON) \
	 DIFF_SOLVER_PATH=$(bsdir)/installed/lib/ \
	 EQUI_LINE_PERCENT1=$(equi_line_percent_inner) \
	 EQUI_LINE_PERCENT2=$(equi_line_percent_outer) \
	 PROB_THRESHOLD=$(prob_threshold) \
	 VOID_STDDEV_FRAC_CRIT=$(void_stddev_frac_crit) \
	 VOID_DIFF_FRAC_CRIT=$(void_diff_frac_crit) \
	 VOID_STDDEV_FRAC_CRIT_THETA=$(void_stddev_frac_crit_theta) \
	 VOID_DIFF_FRAC_CRIT_THETA=$(void_diff_frac_crit_theta) \
	 VOID_STDDEV_FRAC_CRIT_SIGMA=$(void_stddev_frac_crit_sigma) \
	 VOID_DIFF_FRAC_CRIT_SIGMA=$(void_diff_frac_crit_sigma) \
	 VOID_GAMMA_FRAC_SLOPE_MAX_PERCENTILE=$(void_gamma_frac_slope_max_percentile) \
	 COSMIC_RAY_PERCENTILE=$(cosmic_ray_percentile) \
	 N_AVERAGE=$(N_average) \
	 N_NEIGHBOURS=$(N_neighbours) \
	 N_SIGNIFICANCE=$(N_significance) \
	 TOLERANCE_FRAC_STD=$(tolerance_frac_std) \
	 TOLERANCE_FRAC_MEAN=$(tolerance_frac_mean) \
	 MERGE_DISTANCE=$(merge_distance) \
	 STD_XZ_VOID_PROB=$(std_xz_void_prob) \
	 STD_R_VOID_PROB=$(std_R_void_prob) \
	 SPHERICAL_CALC=$(spherical_calc) \
	 TEST_WEYL=$(do_test_weyl) \
	 VELOCITY_CENTRE=$(do_velocity_centre) \
	 MASS_DEF=$(do_mass_def) \
	 ISW=$(do_ISW) \
	 DOPPLER_LENSING=$(do_doppler_lensing) \
	 OPTICAL_SCALARS=$(do_solve_scalars) \
	 STACKING=$(stacking) \
	 NSTACK=$${NSTACK} \
	 STACK_DIR=$(stack_res_dir) \
	 NCPU_LENSING=$(NCPU_lensing) \
	 N_PIXELS=$(N_pixels) \
	 N_SUBSAMPLE_POTENTIAL=$(N_subsample_potential) \
	 REL_ERR_GSL=$(rel_err_gsl) \
	 LENSING_SEED_SAMPLE_POTENTIALS=$(lensing_seed_sample_potentials) \
	 LENSING_SEED_SAMPLE_VELOCITIES=$(lensing_seed_sample_velocities) \
	 run-lensing
	#if [ "x$(stacking)" = "xNO" ] ; then
	for fig in $(output_eps_files); do
	  cp -pv $${fig}_0.eps $(fig_dir)/$${fig}.eps
	  done
	cp -pv rads_vs_prob_mass_def.eps $(fig_dir)
	cp -pv rads_vs_prob_expansion.eps $(fig_dir)
	cp -pv rads_vs_prob_sigma.eps $(fig_dir)
	cp -pv rads_vs_prob_gamma.eps $(fig_dir)
	cp -pv rads_vs_prob_mass_def_projected_void.eps $(fig_dir)
	cp -pv rads_vs_prob_expansion_projected_void.eps $(fig_dir)
	cp -pv rads_vs_prob_sigma_projected_void.eps $(fig_dir)
	cp -pv rads_vs_prob_gamma_projected_void.eps $(fig_dir)
	cp -pv void_populationexpansion.eps $(fig_dir)
	cp -pv void_populationgamma.eps $(fig_dir)
	cp -pv void_populationmass_def.eps $(fig_dir)
	cp -pv void_populationsigma.eps $(fig_dir)
	#fi
	touch $(done_lensing_file)
	printf "The lensing calculations run successfully. See $(lensing_dir) .\n"

# TODO: there is no file with data so far, we only produce plots to develop our
# intuition/physical understanding.
# This section has to be modified as soon as we will produce data.
$(mtexdir)/lensing.tex: $(done_lensing_file)
        # This section produces a file with LaTeX macros for some of
        # the parameters in $(lensing_paramsfile). It presently
        # selects a small hardwired list of parameters rather than all
        # of them. This section will be re-run if $(lensing_paramsfile) is
        # updated.
	printf "%% Automatically produced file.\n\n"
	printf "%% Automatically produced file.\n\n" > $@
	#if [ "x$(stacking)" = "xNO" ] ; then
	param_list_sim="main_void_redshift"
	param_list_dist="observer_void_distance"
	param_list_lens="prob_threshold tolerance_frac_std tolerance_frac_mean void_stddev_frac_crit void_diff_frac_crit std_R_void_prob"
	param_list_lens_float1="merge_distance std_xz_void_prob"
	param_list_lens_float0="cosmic_ray_percentile"
	param_list_lens_int="N_pixels N_average N_neighbours N_significance"
	results_list="N_void N_average_pixel" # N_void_revolver_matched N_void_mass_def_matched N_void_matched"
	results_list_float="matched_fraction matched_mass_def_fraction"
	#results_redshift_list="centre_redshift redshift_std"
	identification_list="matched_voids failed_match original_match"
	$(call create_tex_macro, $(sim_paramsfile), $${param_list_sim}, float.1, )
	$(call create_tex_macro, $(lensing_dir)/observer_void_distance.dat, $${param_list_dist}, float.1, )
	$(call create_tex_macro, $(lensing_paramsfile), $${param_list_lens}, float, )
	$(call create_tex_macro, $(lensing_paramsfile), $${param_list_lens_float1}, float.1, )
	$(call create_tex_macro, $(lensing_paramsfile), $${param_list_lens_float0}, float.0, )
	$(call create_tex_macro, $(lensing_paramsfile), $${param_list_lens_int}, int, )
	$(call create_tex_macro, $(lensing_dir)/mass_def_0.dat, $${results_list}, int, MassDef)
	$(call create_tex_macro, $(lensing_dir)/optical_scalars_0.dat, $${results_list}, int, OS)
	$(call create_tex_macro, $(lensing_dir)/mass_def_0.dat, $${results_list_float}, float.3, MassDef)
	$(call create_tex_macro, $(lensing_dir)/optical_scalars_0.dat, $${results_list_float}, float.3, OS)
	$(call create_tex_macro, $(lensing_dir)/identification.dat, $${identification_list}, int, )
	#$(call create_tex_macro, $(lensing_dir)/velocities.dat, $${results_list}, float, Vel)
	#$(call create_tex_macro, $(lensing_dir)/ISW.dat, $${results_list}, float, ISW)
	#$(call create_tex_macro, $(lensing_dir)/gal_redshift.dat, $${results_redshfit_list}, float.4, )
	#fi
	printf "\nThe LaTeX macro file $@ was created.\n\n"


###full-calculations: $(example_sims_output_file) $(done_run_simulation_file) $(done_detect_haloes_file)

## Run all three independent parts of the calculations.
##
## The simulations part needs $(example_sims_output_file) to be given
## explicitly, because that is not run automatically by later steps in the
## pipeline. The $(done_run_simulation_file) target will wait until
## the simulations have been run before being completed, unless there
## is a timeout.

.PHONY : full-calculations
full-calculations: $(example_sims_output_file) $(done_run_simulation_file) \
              $(done_create_galaxies_file) \
              $(done_detect_voids_file)

## Run the calculations and make the plots.
.PHONY : full-pipeline
full-pipeline: $(example_sims_output_file) $(done_run_simulation_file) \
              $(done_create_galaxies_file) \
              $(done_detect_voids_file) \
              $(done_analyse_plot_file)

.PHONY : clean-lensing-calculations
# TODO: does clean the LaTeX macro belong into here?
clean-lensing-calculations:
	rm -fv $(lensing_dir)/*.eps
	rm -fv $(done_lensing_file)

