#!/usr/bin/env python

import sys
from scipy.spatial import Voronoi
from scipy.spatial import ConvexHull
import numpy as np
from optparse import OptionParser
from read import Read

#buffer_size = 5 #Mpc/h

analyse_snap=int(sys.argv[1])
box_length=int(sys.argv[2])

parser = OptionParser()
parser.add_option(
    '-t',
    '--tracer_file_base',
    dest='TracerFile',
    default='./out_gadget',
    help='path and filename of tracer file',
    metavar='FILE',
    )

(opt, args) = parser.parse_args()
tracer_file_base = opt.TracerFile

if analyse_snap >= 100:
    tracer_file = tracer_file_base + '%d' % analyse_snap
elif 10 <= analyse_snap < 100:
    tracer_file = tracer_file_base + '0%d' % analyse_snap
elif analyse_snap < 10:
    tracer_file = tracer_file_base + '00%d' % analyse_snap

read=Read()
tracers = read.read_tracers(tracer_file)
N_particles = len(tracers)
mean_density = N_particles/(box_length)**3 # = Npart/Volume => unit: m_DM/(Mpc/h)^3

# Copy tracers to the edges of the box using that it is a 3-torus.
# Doing so avoids having open volumes.
#tracers_right_indices = np.where(tracers[:,0] <= buffer_size)
#tracers_right=np.array(tracers[tracers_right_indices])
#tracers_right[:,0]=tracers_right[:,0] + box_length
#
#tracers_left_indices = np.where(tracers[:,0] >= (box_length-buffer_size))
#tracers_left=np.array(tracers[tracers_left_indices])
#tracers_left[:,0]=tracers_left[:,0] - box_length
#
#tracers_up_indices = np.where(tracers[:,2] <= buffer_size)
#tracers_up=np.array(tracers[tracers_up_indices])
#tracers_up[:,2]=tracers_up[:,2] + box_length
#
#tracers_down_indices = np.where(tracers[:,2] >= (box_length-buffer_size))
#tracers_down=np.array(tracers[tracers_down_indices])
#tracers_down[:,2]=tracers_down[:,2] - box_length
#
#tracers_front_indices = np.where(tracers[:,1] <= buffer_size)
#tracers_front=np.array(tracers[tracers_front_indices])
#tracers_front[:,1]=tracers_front[:,1] + box_length
#
#tracers_back_indices = np.where(tracers[:,1] >= (box_length-buffer_size))
#tracers_back=np.array(tracers[tracers_back_indices])
#tracers_back[:,1]=tracers_back[:,1] - box_length
#
#print("np.max(tracers_back[:,1]): ", np.max(tracers_back[:,1]))
#print("np.max(tracers_front[:,1]): ", np.max(tracers_front[:,1]))
#print("np.max(tracers_up[:,2]): ", np.max(tracers_up[:,2]))
#print("np.max(tracers_down[:,2]): ", np.max(tracers_down[:,2]))
#print("np.max(tracers_right[:,0]): ", np.max(tracers_right[:,0]))
#print("np.max(tracers_left[:,0]): ", np.max(tracers_left[:,0]))

#tracers=np.concatenate((tracers,tracers_right, tracers_left, tracers_up, tracers_down, tracers_front, tracers_back),axis=0)
#print("np.max(tracers) :",np.max(tracers))
#print("np.min(tracers) :",np.min(tracers))


v=Voronoi(tracers)
vol = np.zeros(v.npoints)
for i, reg_num in enumerate(v.point_region):
    indices = v.regions[reg_num]
    # Regions extending to a face (or faces) of the box should in
    # principle be continued through to the opposite face(s), using a
    # buffer region, since the space that we are considering has a 3-torus
    # topology. However, the extra RAM usage does not justify the tiny
    # increase in accuracy, so instead we set the density of these regions
    # to the mean density.
    if -1 in indices:
        vol[i] = 1/mean_density
    else:
        vol[i] = ConvexHull(v.vertices[indices]).volume
dens_voronoi = 1/vol
# Only return particles in the main volume
#densities=dens_voronoi[:Ncroot]
densities=dens_voronoi

np.save("densities_voronoi", densities)
