
# This file contains all the plotting comands for the output figures.
# The name of the function indicates what results are plotted. Each
# routine expects all data as input, no further analysis is done in
# these functions. These routines are used for the lensing results of
# a single void as well as for the stacked voids.

import numpy as np
import matplotlib.pyplot as plt

import os

class figures:

    if os.environ.get('METADATA_CREATOR'):
        metadata_creator = os.environ.get('METADATA_CREATOR')
    else:
        metadata_creator = 'Peper, Bolejko, Roukema 2023'
    metadata = {"Creator": metadata_creator}



    def plot_rads_vs_prob(self, rads, prob, identifier):

        plt.rc('font',size=14, family='serif')
        plt.xlabel(r'$R ~ (\mathrm{Mpc}/h)$')
        plt.ylabel(r'$P$')
        plt.scatter(rads, prob)
        figname="rads_vs_prob_" + identifier + ".eps"
        plt.savefig(figname)
        plt.close()

    def plot_test_weyl_cart(self, x_analyse_grid, z_analyse_grid, WeylTensor1010, WeylTensor1020, signWeyl1010, signWeyl1020, diag1010, diag1010_median, poisson_test):

        x_analyse_grid, z_analyse_grid = np.meshgrid(x_analyse_grid, z_analyse_grid, indexing = 'ij', sparse = False)

        plt.rc('font',size=14, family='serif')
        fig, ax = plt.subplots(1, 1)
        plt.ylabel(r'$z ~ (\mathrm{Mpc}/h)$')
        plt.xlabel(r'$x ~ (\mathrm{Mpc}/h)$')
        c = ax.pcolormesh(x_analyse_grid, z_analyse_grid, WeylTensor1010, vmin=np.percentile(WeylTensor1010,10), vmax=np.percentile(WeylTensor1010,90), cmap='cividis')
        cbar=plt.colorbar(c,ax=ax)
        cbar.set_label(r'$C_{1010}$', rotation = 270, labelpad=15)
        figname="C1010_cart_void_" + "gals" + ".eps"
        plt.savefig(figname)
        plt.close()

        plt.rc('font',size=14, family='serif')
        fig, ax = plt.subplots(1, 1)
        plt.ylabel(r'$z ~ (\mathrm{Mpc}/h)$')
        plt.xlabel(r'$x ~ (\mathrm{Mpc}/h)$')
        c = ax.pcolormesh(x_analyse_grid, z_analyse_grid, WeylTensor1020, vmin=np.percentile(WeylTensor1020,10), vmax=np.percentile(WeylTensor1020,90), cmap='cividis')
        cbar=plt.colorbar(c,ax=ax)
        cbar.set_label(r'$C_{1020}$', rotation = 270, labelpad=15)
        figname="C1020_cart_void_" + "gals" + ".eps"
        plt.savefig(figname)
        plt.close()

        plt.rc('font',size=14, family='serif')
        fig, ax = plt.subplots(1, 1)
        plt.ylabel(r'$z ~ (\mathrm{Mpc}/h)$')
        plt.xlabel(r'$x ~ (\mathrm{Mpc}/h)$')
        c = ax.pcolormesh(x_analyse_grid, z_analyse_grid, signWeyl1010, vmin=np.min(signWeyl1010), vmax=np.max(signWeyl1010), cmap='cividis')
        cbar=plt.colorbar(c,ax=ax)
        cbar.set_label(r'sign($C_{1010}$)', rotation = 270, labelpad=15)
        figname="Sign_C1010_cart_void_" + "gals" + ".eps"
        plt.savefig(figname)
        plt.close()

        plt.rc('font',size=14, family='serif')
        fig, ax = plt.subplots(1, 1)
        plt.ylabel(r'$z ~ (\mathrm{Mpc}/h)$')
        plt.xlabel(r'$x ~ (\mathrm{Mpc}/h)$')
        c = ax.pcolormesh(x_analyse_grid, z_analyse_grid, signWeyl1020, vmin=np.min(signWeyl1020), vmax=np.max(signWeyl1020), cmap='cividis')
        cbar=plt.colorbar(c,ax=ax)
        cbar.set_label(r'sign($C_{1020}$)', rotation = 270, labelpad=15)
        figname="Sign_C1020_cart_void" + "gals" + ".eps"
        plt.savefig(figname)
        plt.close()

        plt.rc('font',size=14, family='serif')
        fig, ax = plt.subplots(1, 1)
        plt.ylabel(r'$z ~ (\mathrm{Mpc}/h)$')
        plt.xlabel(r'$x ~ (\mathrm{Mpc}/h)$')
        c = ax.pcolormesh(x_analyse_grid, z_analyse_grid, diag1010, vmin=np.percentile(diag1010,15), vmax= np.percentile(diag1010,85), cmap='cividis')
        cbar=plt.colorbar(c,ax=ax)
        cbar.set_label(r'diag1010', rotation = 270, labelpad=15)
        figname="diag_cart_void" + "gals" + ".eps"
        plt.savefig(figname)
        plt.close()


        plt.rc('font',size=14, family='serif')
        fig, ax = plt.subplots(1, 1)
        plt.ylabel(r'$z ~ (\mathrm{Mpc}/h)$')
        plt.xlabel(r'$x ~ (\mathrm{Mpc}/h)$')
        c = ax.pcolormesh(x_analyse_grid, z_analyse_grid, diag1010_median, cmap='cividis')
        cbar=plt.colorbar(c,ax=ax)
        cbar.set_label(r'diag1010_median', rotation = 270, labelpad=15)
        figname="diag_median_cart_void" + "gals" + ".eps"
        plt.savefig(figname)
        plt.close()

        plt.rc('font',size=14, family='serif')
        fig, ax = plt.subplots(1, 1)
        plt.ylabel(r'$z ~ (\mathrm{Mpc}/h)$')
        plt.xlabel(r'$x ~ (\mathrm{Mpc}/h)$')
        c = ax.pcolormesh(x_analyse_grid, z_analyse_grid, poisson_test, vmin=np.percentile(poisson_test,10), vmax=np.percentile(poisson_test,90), cmap='cividis')
        cbar=plt.colorbar(c,ax=ax)
        cbar.set_label(r'$\Delta \phi$', rotation = 270, labelpad=15)
        figname="poisson_cart_void" + "gals" + ".eps"
        plt.savefig(figname)
        plt.close()


    def plot_vels_cart(self, x_analyse_grid, z_analyse_grid, pixel_radial_vel, pixel_vel_magnitude, pixel_vel_los, pixel_vel_y, centre, void_count):

        x_analyse_grid, z_analyse_grid = np.meshgrid(x_analyse_grid, z_analyse_grid, indexing = 'ij', sparse = False)

        fig, ax = plt.subplots(1, 1)
        plt.rcParams.update({'font.size': 14,
                         'font.family': 'serif'})
        plt.ylabel(r'$z ~ (\mathrm{Mpc}/h)$')
        plt.xlabel(r'$x ~ (\mathrm{Mpc}/h)$')
        c = ax.pcolormesh(x_analyse_grid, z_analyse_grid, pixel_radial_vel, vmin=np.percentile(pixel_radial_vel,10), vmax=np.percentile(pixel_radial_vel,90), cmap='cividis')
        plt.scatter(centre[1][0], centre[1][1], marker='x', s=(56*np.ones(len(centre[1]))), color='red', label='centre')
        cbar=plt.colorbar(c,ax=ax)
        cbar.set_label(r'$\rho / m_{DM}$', rotation = 270, labelpad=15)
        figname=("radial_vel_void_cart_" + "gals_%i" + ".eps") % void_count
        plt.savefig(figname)
        plt.close()

        fig, ax = plt.subplots(1, 1)
        plt.rcParams.update({'font.size': 14,
                         'font.family': 'serif'})
        plt.ylabel(r'$z ~ (\mathrm{Mpc}/h)$')
        plt.xlabel(r'$x ~ (\mathrm{Mpc}/h)$')
        c = ax.pcolormesh(x_analyse_grid, z_analyse_grid, pixel_vel_magnitude, vmin=np.percentile(pixel_vel_magnitude,10), vmax=np.percentile(pixel_vel_magnitude,90), cmap='cividis')
        cbar=plt.colorbar(c,ax=ax)
        cbar.set_label(r'$v$', rotation = 270, labelpad=15)
        figname=("vel_magnitude_void_cart_" + "gals_%i" + ".eps") % void_count
        plt.savefig(figname)
        plt.close()

        fig, ax = plt.subplots(1, 1)
        plt.rcParams.update({'font.size': 14,
                         'font.family': 'serif'})
        plt.ylabel(r'$z ~ (\mathrm{Mpc}/h)$')
        plt.xlabel(r'$x ~ (\mathrm{Mpc}/h)$')
        c = ax.pcolormesh(x_analyse_grid, z_analyse_grid, pixel_vel_los, vmin=np.percentile(pixel_vel_los,10), vmax=np.percentile(pixel_vel_los,90), cmap='cividis')
        plt.scatter(centre[0][0], centre[0][1], marker='x', s=(56*np.ones(len(centre[0]))), color='red', label='centre')
        cbar=plt.colorbar(c,ax=ax)
        cbar.set_label(r'$v_{\mathrm{los}}$', rotation = 270, labelpad=15)
        figname=("vel_los_void_cart_" + "gals_%i" + ".eps") % void_count
        plt.savefig(figname)
        plt.close()

        fig, ax = plt.subplots(1, 1)
        plt.rcParams.update({'font.size': 14,
                         'font.family': 'serif'})
        plt.ylabel(r'$z ~ (\mathrm{Mpc}/h)$')
        plt.xlabel(r'$x ~ (\mathrm{Mpc}/h)$')
        c = ax.pcolormesh(x_analyse_grid, z_analyse_grid, pixel_vel_y, vmin=np.percentile(pixel_vel_y,10), vmax=np.percentile(pixel_vel_y,90), cmap='cividis')
        cbar=plt.colorbar(c,ax=ax)
        cbar.set_label(r'$v_y$', rotation = 270, labelpad=15)
        figname=("vel_y_void_cart_" + "gals_%i" + ".eps") % void_count
        plt.savefig(figname)
        plt.close()


    def plot_density_pot_slice(self, x_analyse_grid, z_analyse_grid, dens, pixel_pot, moved_gals, void_gal_ids, void_count):

        x_analyse_grid, z_analyse_grid = np.meshgrid(x_analyse_grid, z_analyse_grid, indexing = 'ij', sparse = False)


        gal_in_slice_nonvoid_ids = np.delete(np.arange(0,len(moved_gals)),void_gal_ids)
        gal_in_slice_nonvoid_index=np.where(abs(0 - moved_gals[gal_in_slice_nonvoid_ids,1]) <= 5)[0]
        gal_in_slice_nonvoid_ids=gal_in_slice_nonvoid_ids[gal_in_slice_nonvoid_index]
        void_gal_in_slice_index=np.where(abs(0 - moved_gals[void_gal_ids,1]) <= 5)[0]
        void_gal_in_slice_ids=void_gal_ids[void_gal_in_slice_index]


        fig, ax = plt.subplots(1, 1)
        plt.rcParams.update({'font.size': 14,
                         'font.family': 'serif'})
        plt.ylabel(r'$z ~ (\mathrm{Mpc}/h)$')
        plt.xlabel(r'$x ~ (\mathrm{Mpc}/h)$')
        c = ax.pcolormesh(x_analyse_grid, z_analyse_grid, dens, vmax=np.percentile(dens,98), shading='auto', cmap='cividis')
        #void_gal_plot=plt.scatter(moved_gals[void_gal_in_slice_ids,0], moved_gals[void_gal_in_slice_ids,2],
        #                          marker="o", color="blue", s=1, label="void galaxies")
        #nonvoid_gal_plot=plt.scatter(moved_gals[gal_in_slice_nonvoid_ids,0], moved_gals[gal_in_slice_nonvoid_ids,2],
        #                             marker="o", color="red", s=1, label="non-void galaxies")
        #void_gal_plot.set_alpha(1)
        #nonvoid_gal_plot.set_alpha(1)

        cbar=plt.colorbar(c,ax=ax)
        #equi_pixel = ax.pcolormesh(azimuth, polar, random_pixel_pot_value10, cmap='GnBu')
        #equi_pixel = ax.pcolormesh(azimuth, polar, random_pixel_pot_value30, cmap='GnBu')
        cbar.set_label(r'$\rho ~~ (m_{DM}/(\mathrm{Mpc}/h)^3)$', rotation = 270, labelpad=20)
        figname=("densities_cart_void_" + "gals_%i" + ".eps") % void_count
        plt.savefig(figname)
        plt.close()

        fig, ax = plt.subplots(1, 1)
        plt.rcParams.update({'font.size': 14,
                         'font.family': 'serif'})
        plt.ylabel(r'$z ~ (\mathrm{Mpc}/h)$')
        plt.xlabel(r'$x ~ (\mathrm{Mpc}/h)$')
        c = ax.pcolormesh(x_analyse_grid, z_analyse_grid, pixel_pot, vmin=np.percentile(pixel_pot,5), vmax=np.percentile(pixel_pot,95), cmap='cividis')
        cbar=plt.colorbar(c,ax=ax)
        cbar.set_label(r'$\phi$', rotation = 270, labelpad=15)
        figname=("potential_cart_void_" + "gals_%i" + ".eps") % void_count
        plt.savefig(figname)
        plt.close()

    def plot_mass_def_cart(self, x_analyse_grid, z_analyse_grid, mass_def, centre, rads, average_mass_def_dens, average_mass_def_dens_cut, average_mass_def, average_mass_def_gamma_centre, mass_def_rad, moved_revolver_centres, chosen_void_id, box_length, void_count):

        x_analyse_grid, z_analyse_grid = np.meshgrid(x_analyse_grid, z_analyse_grid, indexing = 'ij', sparse = False)

        ## Plot a map of the projected mass deficit Sigma.
        plt.rc('font',size=14, family='serif')
        fig, ax = plt.subplots(1, 1)
        plt.ylabel(r'$z ~ (\mathrm{Mpc}/h)$')
        plt.xlabel(r'$x ~ (\mathrm{Mpc}/h)$')
        plt.xlim([-(box_length/2), (box_length/2)])
        plt.ylim([-(box_length/2), (box_length/2)])
        # Select the inner 80-percentile range to get a visually useful map
        c = ax.pcolormesh(x_analyse_grid, z_analyse_grid, mass_def, vmin=np.percentile(mass_def,2), vmax=np.percentile(mass_def,98), cmap='cividis')
        cbar=plt.colorbar(c,ax=ax)
        #equi_pixel = ax.pcolormesh(azimuth, polar, random_pixel_value10, cmap='GnBu')
        #equi_pixel = ax.pcolormesh(azimuth, polar, random_pixel_value30, cmap='GnBu')
        cbar.set_label(r'$\Sigma ~ (m_{\mathrm{DM}}/(\mathrm{Mpc}/h)^2)$', rotation = 270, labelpad=20)
        plt.scatter(moved_revolver_centres[:,0], moved_revolver_centres[:,2], marker='+', s=(56*np.ones(len(moved_revolver_centres))), color='white')
        #plt.scatter(moved_revolver_centres[chosen_void_id,0], moved_revolver_centres[chosen_void_id,2], marker='x', color='magenta')
        #plt.scatter(0, 0, marker='x', color='black', label='geom centre')
        plt.scatter(centre[:,0], centre[:,1], marker='x', s=(56*np.ones(len(centre))), color='red', label='geom centre')
        for i in range(0, len(mass_def_rad)):
            circle_rad = plt.Circle((centre[i,0], centre[i,1]), mass_def_rad[i], color='red', fill=False)
            ax.add_patch(circle_rad)
        figname=("mass_deficit_void_cart_" + "gals_%i" + ".eps") % void_count
        plt.savefig(figname)
        plt.close()

        plt.rc('font',size=14, family='serif')
        fig, ax = plt.subplots(1, 1)
        plt.ylabel(r'$z ~ (\mathrm{Mpc}/h)$')
        plt.xlabel(r'$x ~ (\mathrm{Mpc}/h)$')
        # Select the full range of values - this map is called "no_cut" and tends to be less useful
        c = ax.pcolormesh(x_analyse_grid, z_analyse_grid, mass_def, cmap='cividis')
        cbar=plt.colorbar(c,ax=ax)
        #equi_pixel = ax.pcolormesh(azimuth, polar, random_pixel_value10, cmap='GnBu')
        #equi_pixel = ax.pcolormesh(azimuth, polar, random_pixel_value30, cmap='GnBu')
        cbar.set_label(r'$\Sigma ~ (m_{\mathrm{DM}}/(\mathrm{Mpc}/h)^2)$', rotation = 270, labelpad=20)
        figname=("mass_deficit_no_cut_void_cart_" + "gals_%i" + ".eps") % void_count
        plt.tight_layout()
        plt.savefig(figname, metadata = self.metadata)
        plt.close()


        norm_rads=np.linspace(0.1, 1.5,101) # normalised radii
        average_mass_def_mean=np.mean(average_mass_def, axis=0) # 2D voids - find the mean
        average_mass_def_std=np.std(average_mass_def, axis=0)
        average_mass_def_dens_mean=np.mean(average_mass_def_dens, axis=0) # profiles using 3D void centres and radial normalisation - find the mean
        average_mass_def_dens_std=np.std(average_mass_def_dens, axis=0)
        average_mass_def_dens_cut_mean=np.mean(average_mass_def_dens_cut, axis=0)
        average_mass_def_dens_cut_std=np.std(average_mass_def_dens_cut, axis=0)
        average_mass_def_gamma_centre_mean=np.mean(average_mass_def_gamma_centre, axis=0)
        average_mass_def_gamma_centre_std=np.std(average_mass_def_gamma_centre, axis=0)

        # Not so useful: Normalised profiles of voids bigger than a radius defined in the calling routine (e.g. in lensing.py)
        plt.rc('font',size=14, family='serif')
        plt.style.use('tableau-colorblind10')
        fig, ax = plt.subplots(1, 1)
        plt.plot(norm_rads, average_mass_def_dens_mean, color='blue', label='Revolver centre')
        plt.fill_between(norm_rads, (average_mass_def_dens_mean-average_mass_def_dens_std),
                         (average_mass_def_dens_mean+average_mass_def_dens_std), color='none', edgecolor='cyan', hatch="\\\\")
        plt.plot(norm_rads, average_mass_def_dens_cut_mean, color='red', label='Revolver centre cut')
        plt.fill_between(norm_rads, (average_mass_def_dens_cut_mean-average_mass_def_dens_cut_std),
                         (average_mass_def_dens_cut_mean+average_mass_def_dens_cut_std), color='none', edgecolor='tomato', hatch="//")
        #plt.plot(rads, average_mass_def_gamma_centre_mean, 'b-', label='Gamma centre')
        #plt.fill_between(rads, (average_mass_def_gamma_centre_mean-average_mass_def_gamma_centre_std),
        #                 (average_mass_def_gamma_centre_mean+average_mass_def_gamma_centre_std)  ,alpha=0.3, facecolor="blue")

        #plt.plot(self.rads, average_mass_def_combined, 'g-', label='Combined centre')
        plt.grid()
        plt.legend(loc='best')
        plt.xlabel(r'$r/R_\mathrm{void}$')
        plt.ylabel(r'$\overline{\Sigma} ~ (m_{\mathrm{DM}}/(\mathrm{Mpc}/h)^2)$')
        figname=("average_mass_def_revolver_%i" + ".eps") % void_count
        plt.tight_layout()
        plt.savefig(figname, format = 'eps', metadata = self.metadata)
        plt.close()


        # More useful: Normalised profiles for full selections of 2D and 3D voids
        plt.rc('font',size=14, family='serif')
        plt.style.use('tableau-colorblind10')
        fig, ax = plt.subplots(1, 1)
        # 3D voids
        plt.plot(norm_rads, average_mass_def_dens_mean, color='blue', label='Revolver centres')
        plt.fill_between(norm_rads, (average_mass_def_dens_mean-average_mass_def_dens_std),
                         (average_mass_def_dens_mean+average_mass_def_dens_std), color='none', edgecolor='cyan', hatch="\\\\")
        # 2D voids
        plt.plot(norm_rads, average_mass_def_mean, color='red', label=(r'$\overline{\Sigma}$ centres'))
        plt.fill_between(norm_rads, (average_mass_def_mean-average_mass_def_std),
                         (average_mass_def_mean+average_mass_def_std), color='none', edgecolor='tomato', hatch="//")
        #plt.plot(rads, average_mass_def_gamma_centre_mean, 'b-', label='Gamma centre')
        #plt.fill_between(rads, (average_mass_def_gamma_centre_mean-average_mass_def_gamma_centre_std),
        #                 (average_mass_def_gamma_centre_mean+average_mass_def_gamma_centre_std)  ,alpha=0.3, facecolor="blue")

        #plt.plot(self.rads, average_mass_def_combined, 'g-', label='Combined centre')
        plt.grid()
        plt.legend(loc='best')
        plt.xlabel(r'$r/R_\mathrm{void}$')
        plt.ylabel(r'$\overline{\Sigma} ~ (m_{\mathrm{DM}}/(\mathrm{Mpc}/h)^2)$')
        figname=("average_mass_def_%i" + ".eps") % void_count
        plt.tight_layout()
        plt.savefig(figname, format = 'eps', metadata = self.metadata)
        plt.close()


    def plot_weak_lensing_cart(self, x_analyse_grid, z_analyse_grid, kappa_grid, gamma_grid, centre, gamma_rads, moved_revolver_centres, average_gamma, average_gamma_revolver, average_gamma_revolver_cut, box_length, void_count):
        x_analyse_grid, z_analyse_grid = np.meshgrid(x_analyse_grid, z_analyse_grid, indexing = 'ij', sparse = False)

        plt.rc('font',size=14, family='serif')
        fig, ax = plt.subplots(1, 1)
        plt.ylabel(r'$z ~ (\mathrm{Mpc}/h)$')
        plt.xlabel(r'$x ~ (\mathrm{Mpc}/h)$')
        c = ax.pcolormesh(x_analyse_grid, z_analyse_grid, kappa_grid, vmin=np.percentile(kappa_grid,10), vmax=np.percentile(kappa_grid,90), cmap='cividis')
        cbar=plt.colorbar(c,ax=ax)
        cbar.set_label(r'$\kappa$', rotation = 270, labelpad=15)
        figname=("weak_lensing_kappa_grid_" + "gals_%i" + ".eps") % void_count
        plt.savefig(figname)
        plt.close()

        plt.rc('font',size=14, family='serif')
        fig, ax = plt.subplots(1, 1)
        plt.ylabel(r'$z ~ (\mathrm{Mpc}/h)$')
        plt.xlabel(r'$x ~ (\mathrm{Mpc}/h)$')
        plt.xlim([-(box_length/2), (box_length/2)])
        plt.ylim([-(box_length/2), (box_length/2)])
        c = ax.pcolormesh(x_analyse_grid, z_analyse_grid, gamma_grid, vmin=np.percentile(gamma_grid,10), vmax=np.percentile(gamma_grid,90), cmap='cividis')
        cbar=plt.colorbar(c,ax=ax)
        plt.scatter(moved_revolver_centres[:,0], moved_revolver_centres[:,2], marker='+', s=(56*np.ones(len(moved_revolver_centres))), color='white')
        plt.scatter(centre[:,0], centre[:,1], marker='x', s=(56*np.ones(len(centre))), color='red', label='centre')
        for i in range(0, len(gamma_rads)):
            circle_rad = plt.Circle((centre[i,0], centre[i,1]), gamma_rads[i], color='red', fill=False)
            ax.add_patch(circle_rad)
        cbar.set_label(r'$\gamma$', rotation = 270, labelpad=15)
        figname=("weak_lensing_gamma_grid_" + "gals_%i" + ".eps") % void_count
        plt.savefig(figname)
        plt.close()


        norm_rads=np.linspace(0.1, 1.5,101)
        average_gamma_mean=np.mean(average_gamma, axis=0)
        average_gamma_std=np.std(average_gamma, axis=0)
        average_gamma_revolver_mean=np.mean(average_gamma_revolver, axis=0)
        average_gamma_revolver_std=np.std(average_gamma_revolver, axis=0)
        average_gamma_revolver_cut_mean=np.mean(average_gamma_revolver_cut, axis=0)
        average_gamma_revolver_cut_std=np.std(average_gamma_revolver_cut, axis=0)

        #plt.plot(rads, average_exp_dens, 'k-', label='Revolver centre')
        #plt.plot(rads, average_exp_centre, 'r-', label='Expansion centre')
        plt.rc('font',size=14, family='serif')
        fig, ax = plt.subplots(1, 1)
        plt.style.use('tableau-colorblind10')
        plt.ylim([-1.e-6, 1.e-6])
        plt.plot(norm_rads, average_gamma_revolver_mean, color='blue', label='Revolver centre')
        plt.fill_between(norm_rads, (average_gamma_revolver_mean-average_gamma_revolver_std),
                         (average_gamma_revolver_mean+average_gamma_revolver_std), color='none', edgecolor='cyan', hatch="\\\\")
        plt.plot(norm_rads, average_gamma_revolver_cut_mean, color='red', label='Revolver centre cut')
        plt.fill_between(norm_rads, (average_gamma_revolver_cut_mean-average_gamma_revolver_cut_std),
                         (average_gamma_revolver_cut_mean+average_gamma_revolver_cut_std), color='none', edgecolor='tomato', hatch="//")
        plt.grid()
        plt.legend(loc='best')
        plt.xlabel(r'$r/R_\mathrm{void}$')
        plt.ylabel(r'$\overline{\gamma}$')
        figname=("average_weak_lensing_gamma_revolver_%i" + ".eps") % void_count
        plt.tight_layout()
        plt.savefig(figname, format = 'eps', metadata = self.metadata)
        plt.close()


        plt.rc('font',size=14, family='serif')
        fig, ax = plt.subplots(1, 1)
        plt.style.use('tableau-colorblind10')
        plt.ylim([-1.e-6, 1.e-6])
        plt.plot(norm_rads, average_gamma_revolver_mean, color='blue', label='Revolver centres')
        plt.fill_between(norm_rads, (average_gamma_revolver_mean-average_gamma_revolver_std),
                         (average_gamma_revolver_mean+average_gamma_revolver_std), color='none', edgecolor='cyan', hatch="\\\\")
        plt.plot(norm_rads, average_gamma_mean, color='red', label=(r'$\overline{\gamma_{\perp}}$ centres'))
        plt.fill_between(norm_rads, (average_gamma_mean-average_gamma_std),
                         (average_gamma_mean+average_gamma_std), color='none', edgecolor='tomato', hatch="//")
        plt.grid()
        plt.legend(loc='best')
        plt.xlabel(r'$r/R_\mathrm{void}$')
        plt.ylabel(r'$\overline{\gamma}$')
        figname=("average_weak_lensing_gamma_%i" + ".eps") % void_count
        plt.tight_layout()
        plt.savefig(figname, format = 'eps', metadata = self.metadata)
        plt.close()


    def plot_ISW_cart(self, x_analyse_grid, z_analyse_grid, ISW, pot_deriv, centre, maxima, void_count):

        x_analyse_grid, z_analyse_grid = np.meshgrid(x_analyse_grid, z_analyse_grid, indexing = 'ij', sparse = False)

        fig, ax = plt.subplots(1, 1)
        plt.rcParams.update({'font.size': 14,
                         'font.family': 'serif'})
        plt.ylabel(r'$z ~ (\mathrm{Mpc}/h)$')
        plt.xlabel(r'$x ~ (\mathrm{Mpc}/h)$')
        c = ax.pcolormesh(x_analyse_grid, z_analyse_grid, ISW, vmin=np.percentile(ISW,10), vmax=np.percentile(ISW,90), cmap='cividis')
        cbar=plt.colorbar(c,ax=ax)
        plt.scatter(maxima[0], maxima[1], marker='x', s=(56*np.ones(len(maxima))), color='green', label='all maxima')
        plt.scatter(centre[0], centre[1], marker='x', s=(56*np.ones(len(centre))), color='red', label='geom centre')
        cbar.set_label(r'$\Delta T ~ (K)$', rotation = 270, labelpad=15)
        figname=("ISW_void_cart_" + "gals_%i" + ".eps") % void_count
        plt.savefig(figname)
        plt.close()

        fig, ax = plt.subplots(1, 1)
        plt.rcParams.update({'font.size': 14,
                         'font.family': 'serif'})
        plt.ylabel(r'$z ~ (\mathrm{Mpc}/h)$')
        plt.xlabel(r'$x ~ (\mathrm{Mpc}/h)$')
        c = ax.pcolormesh(x_analyse_grid, z_analyse_grid, pot_deriv, vmin=np.percentile(pot_deriv,10), vmax=np.percentile(pot_deriv,90), cmap='cividis')
        cbar=plt.colorbar(c,ax=ax)
        cbar.set_label(r'$\dot{\Phi}$', rotation = 270, labelpad=15)
        figname=("time_deriv_pot_void_cart_" + "gals_%i" + ".eps") % void_count
        plt.savefig(figname)
        plt.close()


    def plot_doppler_cart(self, x_analyse_grid, z_analyse_grid, doppler_convergence):


        max_difference=doppler_convergence[:,:,4]-doppler_convergence[:,:,0]

        x_analyse_grid, z_analyse_grid = np.meshgrid(x_analyse_grid, z_analyse_grid, indexing = 'ij', sparse = False)

        fig, ax = plt.subplots(1, 1)
        plt.rcParams.update({'font.size': 14,
                         'font.family': 'serif'})
        plt.ylabel(r'$z ~ (\mathrm{Mpc}/h)$')
        plt.xlabel(r'$x ~ (\mathrm{Mpc}/h)$')
        c = ax.pcolormesh(x_analyse_grid, z_analyse_grid, doppler_convergence[:,:,0], vmin=np.percentile(doppler_convergence[:,:,0],10), vmax=np.percentile(doppler_convergence[:,:,0],90), cmap='cividis')
        cbar=plt.colorbar(c,ax=ax)
        cbar.set_label(r'$\kappa$', rotation = 270, labelpad=15)
        figname="doppler_convergence_void_cart_Y-voidrad" + "gals" + ".eps"
        plt.savefig(figname)
        plt.close()

        fig, ax = plt.subplots(1, 1)
        plt.rcParams.update({'font.size': 14,
                         'font.family': 'serif'})
        plt.ylabel(r'$z ~ (\mathrm{Mpc}/h)$')
        plt.xlabel(r'$x ~ (\mathrm{Mpc}/h)$')
        c = ax.pcolormesh(x_analyse_grid, z_analyse_grid, doppler_convergence[:,:,1], vmin=np.percentile(doppler_convergence[:,:,1],10), vmax=np.percentile(doppler_convergence[:,:,1],90), cmap='cividis')
        cbar=plt.colorbar(c,ax=ax)
        cbar.set_label(r'$\kappa$', rotation = 270, labelpad=15)
        figname="doppler_convergence_void_cart_Y-halfrad" + "gals" + ".eps"
        plt.savefig(figname)
        plt.close()

        fig, ax = plt.subplots(1, 1)
        plt.rcParams.update({'font.size': 14,
                         'font.family': 'serif'})
        plt.ylabel(r'$z ~ (\mathrm{Mpc}/h)$')
        plt.xlabel(r'$x ~ (\mathrm{Mpc}/h)$')
        c = ax.pcolormesh(x_analyse_grid, z_analyse_grid, doppler_convergence[:,:,2], vmin=np.percentile(doppler_convergence[:,:,2],10), vmax=np.percentile(doppler_convergence[:,:,2],90), cmap='cividis')
        cbar=plt.colorbar(c,ax=ax)
        cbar.set_label(r'$\kappa$', rotation = 270, labelpad=15)
        figname="doppler_convergence_void_cart_Y0" + "gals" + ".eps"
        plt.savefig(figname)
        plt.close()

        fig, ax = plt.subplots(1, 1)
        plt.rcParams.update({'font.size': 14,
                         'font.family': 'serif'})
        plt.ylabel(r'$z ~ (\mathrm{Mpc}/h)$')
        plt.xlabel(r'$x ~ (\mathrm{Mpc}/h)$')
        c = ax.pcolormesh(x_analyse_grid, z_analyse_grid, doppler_convergence[:,:,3], vmin=np.percentile(doppler_convergence[:,:,3],10), vmax=np.percentile(doppler_convergence[:,:,3],90), cmap='cividis')
        cbar=plt.colorbar(c,ax=ax)
        cbar.set_label(r'$\kappa$', rotation = 270, labelpad=15)
        figname="doppler_convergence_void_cart_Yhalfrad" + "gals" + ".eps"
        plt.savefig(figname)
        plt.close()

        fig, ax = plt.subplots(1, 1)
        plt.rcParams.update({'font.size': 14,
                         'font.family': 'serif'})
        plt.ylabel(r'$z ~ (\mathrm{Mpc}/h)$')
        plt.xlabel(r'$x ~ (\mathrm{Mpc}/h)$')
        c = ax.pcolormesh(x_analyse_grid, z_analyse_grid, doppler_convergence[:,:,4], vmin=np.percentile(doppler_convergence[:,:,4],10), vmax=np.percentile(doppler_convergence[:,:,4],90), cmap='cividis')
        cbar=plt.colorbar(c,ax=ax)
        cbar.set_label(r'$\kappa$', rotation = 270, labelpad=15)
        figname="doppler_convergence_void_cart_Yvoidrad" + "gals" + ".eps"
        plt.savefig(figname)
        plt.close()

        fig, ax = plt.subplots(1, 1)
        plt.rcParams.update({'font.size': 14,
                         'font.family': 'serif'})
        plt.ylabel(r'$z ~ (\mathrm{Mpc}/h)$')
        plt.xlabel(r'$x ~ (\mathrm{Mpc}/h)$')
        c = ax.pcolormesh(x_analyse_grid, z_analyse_grid, max_difference, vmin=np.percentile(max_difference,10), vmax=np.percentile(max_difference,90), cmap='cividis')
        cbar=plt.colorbar(c,ax=ax)
        cbar.set_label(r'$\kappa$', rotation = 270, labelpad=15)
        figname="doppler_convergence_max_diff" + "gals" + ".eps"
        plt.savefig(figname)
        plt.close()


    def plot_optical_scalars_cart(self, x_analyse_grid, z_analyse_grid, rads, real_theta, rel_deviation, real_sigma, img_sigma, distance_check, centre_exp, rads_exp, centre_sigma, rads_sigma, average_exp_dens, average_exp_dens_cut, average_exp_centre, average_sigma_dens, average_sigma_dens_cut, average_sigma_centre, moved_revolver_centres, chosen_void_id_exp, chosen_void_id_sigma, box_length, void_count):

        x_analyse_grid, z_analyse_grid = np.meshgrid(x_analyse_grid, z_analyse_grid, indexing = 'ij', sparse = False)

        plt.rc('font',size=14, family='serif')
        fig, ax = plt.subplots(1, 1)
        plt.ylabel(r'$z ~ (\mathrm{Mpc}/h)$')
        plt.xlabel(r'$x ~ (\mathrm{Mpc}/h)$')
        plt.xlim([-(box_length/2), (box_length/2)])
        plt.ylim([-(box_length/2), (box_length/2)])
        #c = ax.pcolormesh(azimuth, polar, real_theta[:,:,len(real_theta)-1], vmin=np.percentile(real_theta,10), vmax=np.percentile(real_theta,90))
        c = ax.pcolormesh(x_analyse_grid, z_analyse_grid, real_theta, vmin=np.percentile(real_theta,5), vmax=np.percentile(real_theta,95), cmap='cividis')
        plt.scatter(moved_revolver_centres[:,0], moved_revolver_centres[:,2], marker='+', s=(56*np.ones(len(moved_revolver_centres))), color='white')
        plt.scatter(centre_exp[:,0], centre_exp[:,1], color='red', marker='x', s=(56*np.ones(len(centre_exp))))
        for i in range(0, len(rads_exp)):
            circle_rad = plt.Circle((centre_exp[i,0], centre_exp[i,1]), rads_exp[i], color='red', fill=False)
            ax.add_patch(circle_rad)
        #plt.scatter(moved_revolver_centres[chosen_void_id_exp,0], moved_revolver_centres[chosen_void_id_exp,2], marker='x', color='magenta')
        #plt.scatter(0, 0, marker='x', color='black', label='geom centre')
        cbar=plt.colorbar(c,ax=ax)
        cbar.set_label(r'$\theta ~ (1/\mathrm{Mpc})$', rotation = 270, labelpad=15)
        figname=("real_theta_void_cart_" + "gals_%i" + ".eps") % void_count
        plt.savefig(figname)
        plt.close()


        plt.rc('font',size=14, family='serif')
        fig, ax = plt.subplots(1, 1)
        plt.ylabel(r'$z ~ (\mathrm{Mpc}/h)$')
        plt.xlabel(r'$x ~ (\mathrm{Mpc}/h)$')
        #c = ax.pcolormesh(azimuth, polar, real_theta[:,:,len(real_theta)-1], vmin=np.percentile(real_theta,10), vmax=np.percentile(real_theta,90))
        c = ax.pcolormesh(x_analyse_grid, z_analyse_grid, rel_deviation, vmin=np.percentile(rel_deviation,5), vmax=np.percentile(rel_deviation,95), cmap='cividis')
        cbar=plt.colorbar(c,ax=ax)
        cbar.set_label(r'$(\theta - \bar{\theta})/\bar{\theta}$', rotation = 270, labelpad=15)
        figname=("theta_deviation_void_cart_" + "gals_%i" + ".eps") % void_count
        plt.savefig(figname)
        plt.close()

        plt.rc('font',size=14, family='serif')
        fig, ax = plt.subplots(1, 1)
        plt.ylabel(r'$z ~ (\mathrm{Mpc}/h)$')
        plt.xlabel(r'$x ~ (\mathrm{Mpc}/h)$')
        #c = ax.pcolormesh(azimuth, polar, real_sigma[:,:,len(real_sigma)-1], vmin=np.percentile(real_sigma,10), vmax=np.percentile(real_sigma,90))
        c = ax.pcolormesh(x_analyse_grid, z_analyse_grid, real_sigma, vmin=np.percentile(real_sigma,5), vmax=np.percentile(real_sigma,95), cmap='cividis')
        cbar=plt.colorbar(c,ax=ax)
        cbar.set_label(r'$real(\sigma)$', rotation = 270, labelpad=15)
        figname=("real_sigma_void_cart_" + "gals_%i" + ".eps") % void_count
        plt.savefig(figname)
        plt.close()

        plt.rc('font',size=14, family='serif')
        fig, ax = plt.subplots(1, 1)
        plt.ylabel(r'$z ~ (\mathrm{Mpc}/h)$')
        plt.xlabel(r'$x ~ (\mathrm{Mpc}/h)$')
        #c = ax.pcolormesh(azimuth, polar, img_sigma[:,:,len(img_sigma)-1], vmin=np.percentile(img_sigma,10), vmax=np.percentile(img_sigma,90))
        c = ax.pcolormesh(x_analyse_grid, z_analyse_grid, img_sigma, vmin=np.percentile(img_sigma,5), vmax=np.percentile(img_sigma,95), cmap='cividis')
        cbar=plt.colorbar(c,ax=ax)
        cbar.set_label(r'$img(\sigma)$', rotation = 270, labelpad=15)
        figname=("img_sigma_void_cart_" + "gals_%i" + ".eps") % void_count
        plt.savefig(figname)
        plt.close()

        plt.rc('font',size=14, family='serif')
        fig, ax = plt.subplots(1, 1)
        plt.ylabel(r'$z ~ (\mathrm{Mpc}/h)$')
        plt.xlabel(r'$x ~ (\mathrm{Mpc}/h)$')
        #c = ax.pcolormesh(azimuth, polar, img_sigma[:,:,len(img_sigma)-1], vmin=np.percentile(img_sigma,10), vmax=np.percentile(img_sigma,90))
        mag_sigma = np.sqrt((real_sigma**2)+(img_sigma**2))
        fig, ax = plt.subplots(1, 1)
        plt.ylabel(r'$z ~ (\mathrm{Mpc}/h)$')
        plt.xlabel(r'$x ~ (\mathrm{Mpc}/h)$')
        plt.xlim([-(box_length/2), (box_length/2)])
        plt.ylim([-(box_length/2), (box_length/2)])
        #c = ax.pcolormesh(azimuth, polar, img_sigma[:,:,len(img_sigma)-1], vmin=np.percentile(img_sigma,10), vmax=np.percentile(img_sigma,90))
        c = ax.pcolormesh(x_analyse_grid, z_analyse_grid, mag_sigma, vmin=np.percentile(mag_sigma,5), vmax=np.percentile(mag_sigma,95), cmap='cividis')
        cbar=plt.colorbar(c,ax=ax)
        plt.scatter(moved_revolver_centres[:,0], moved_revolver_centres[:,2], marker='+', s=(56*np.ones(len(moved_revolver_centres))), color='white')
        plt.scatter(centre_sigma[:,0], centre_sigma[:,1], color='red', marker='x', s=(56*np.ones(len(centre_sigma))))
        for i in range(0, len(rads_sigma)):
            circle_rad = plt.Circle((centre_sigma[i,0], centre_sigma[i,1]), rads_sigma[i], color='red', fill=False)
            ax.add_patch(circle_rad)
        #plt.scatter(moved_revolver_centres[chosen_void_id_sigma,0], moved_revolver_centres[chosen_void_id_sigma,2], marker='x', color='magenta')
        #plt.scatter(0, 0, marker='x', color='black', label='geom centre')
        cbar.set_label(r'$|\sigma| ~ (1/\mathrm{Mpc})$', rotation = 270, labelpad=15)
        figname=("mag_sigma_void_cart_" + "gals_%i" + ".eps") % void_count
        plt.savefig(figname)
        plt.close()

        plt.rc('font',size=14, family='serif')
        fig, ax = plt.subplots(1, 1)
        plt.ylabel(r'$z ~ (\mathrm{Mpc}/h)$')
        plt.xlabel(r'$x ~ (\mathrm{Mpc}/h)$')
        #c = ax.pcolormesh(azimuth, polar, img_sigma[:,:,len(img_sigma)-1], vmin=np.percentile(img_sigma,10), vmax=np.percentile(img_sigma,90))
        c = ax.pcolormesh(x_analyse_grid, z_analyse_grid, distance_check, vmin=np.percentile(distance_check,10), vmax=np.percentile(distance_check,90), cmap='cividis')
        cbar=plt.colorbar(c,ax=ax)
        cbar.set_label(r'$D$', rotation = 270, labelpad=15)
        figname=("distance_check_void_cart_" + "gals_%i" + ".eps") % void_count
        plt.savefig(figname)
        plt.close()


        norm_rads=np.linspace(0.1, 1.5,101)
        average_exp_mean=np.mean(average_exp_centre, axis=0)
        average_exp_std=np.std(average_exp_centre, axis=0)
        average_exp_dens_mean=np.mean(average_exp_dens, axis=0)
        average_exp_dens_std=np.std(average_exp_dens, axis=0)
        average_exp_dens_cut_mean=np.mean(average_exp_dens_cut, axis=0)
        average_exp_dens_cut_std=np.std(average_exp_dens_cut, axis=0)
        average_sigma_mean=np.mean(average_sigma_centre, axis=0)
        average_sigma_std=np.std(average_sigma_centre, axis=0)
        average_sigma_dens_mean=np.mean(average_sigma_dens, axis=0)
        average_sigma_dens_std=np.std(average_sigma_dens, axis=0)
        average_sigma_dens_cut_mean=np.mean(average_sigma_dens_cut, axis=0)
        average_sigma_dens_cut_std=np.std(average_sigma_dens_cut, axis=0)


        #plt.plot(rads, average_exp_dens, 'k-', label='Revolver centre')
        #plt.plot(rads, average_exp_centre, 'r-', label='Expansion centre')
        plt.rc('font',size=14, family='serif')
        fig, ax = plt.subplots(1, 1)
        plt.style.use('tableau-colorblind10')
        plt.plot(norm_rads, average_exp_dens_mean, color='blue', label='Revolver centre')
        plt.fill_between(norm_rads, (average_exp_dens_mean-average_exp_dens_std),
                         (average_exp_dens_mean+average_exp_dens_std), color='none', edgecolor='cyan', hatch="\\\\")
        plt.plot(norm_rads, average_exp_dens_cut_mean, color='red', label='Revolver centre cut')
        plt.fill_between(norm_rads, (average_exp_dens_cut_mean-average_exp_dens_cut_std),
                         (average_exp_dens_cut_mean+average_exp_dens_cut_std), color='none', edgecolor='tomato', hatch="//")
        plt.grid()
        plt.legend(loc='best')

        plt.xlabel(r'$r/R_\mathrm{void}$')
        plt.ylabel(r'$\overline{\theta} ~ (1/\mathrm{Mpc})$')

        figname=("average_expansion_revolver_%i" + ".eps") % void_count
        plt.tight_layout()
        plt.savefig(figname, format = 'eps', metadata = self.metadata)
        plt.close()

        plt.rc('font',size=14, family='serif')
        fig, ax = plt.subplots(1, 1)
        plt.style.use('tableau-colorblind10')
        plt.plot(norm_rads, average_exp_dens_mean, color='blue', label='Revolver centres')
        plt.fill_between(norm_rads, (average_exp_dens_mean-average_exp_dens_std),
                         (average_exp_dens_mean+average_exp_dens_std), color='none', edgecolor='cyan', hatch="\\\\")
        plt.plot(norm_rads, average_exp_mean, color='red', label=(r'$\overline{\theta}$ centres'))
        plt.fill_between(norm_rads, (average_exp_mean-average_exp_std),
                         (average_exp_mean+average_exp_std), color='none', edgecolor='tomato', hatch="//")
        plt.grid()
        plt.legend(loc='best')

        plt.xlabel(r'$r/R_\mathrm{void}$')
        plt.ylabel(r'$\overline{\theta} ~ (1/\mathrm{Mpc})$')
        figname=("average_expansion_%i" + ".eps") % void_count
        plt.tight_layout()
        plt.savefig(figname, format = 'eps', metadata = self.metadata)
        plt.close()

        #plt.plot(rads, average_sigma_dens, 'k-', label='Revolver centre')
        #plt.plot(rads, average_sigma_centre, 'r-', label='Expansion centre')
        plt.rc('font',size=14, family='serif')
        fig, ax = plt.subplots(1, 1)
        plt.style.use('tableau-colorblind10')
        plt.plot(norm_rads, average_sigma_dens_mean, color='blue', label='Revolver centre')
        plt.fill_between(norm_rads, (average_sigma_dens_mean-average_sigma_dens_std),
                         (average_sigma_dens_mean+average_sigma_dens_std), color='none', edgecolor='cyan', hatch="\\\\")
        plt.plot(norm_rads, average_sigma_dens_cut_mean, color='red', label='Revolver centre cut')
        plt.fill_between(norm_rads, (average_sigma_dens_cut_mean-average_sigma_dens_cut_std),
                         (average_sigma_dens_cut_mean+average_sigma_dens_cut_std), color='none', edgecolor='tomato', hatch="//")
        plt.grid()
        plt.legend(loc='best')
        plt.xlabel(r'$r/R_\mathrm{void}$')
        plt.ylabel(r'$\overline{\vert\sigma\vert} ~ (1/\mathrm{Mpc})$')
        figname=("average_sigma_revolver_%i" + ".eps") % void_count
        plt.tight_layout()
        plt.savefig(figname, format = 'eps', metadata = self.metadata)
        plt.close()


        plt.rc('font',size=14, family='serif')
        fig, ax = plt.subplots(1, 1)
        plt.style.use('tableau-colorblind10')
        plt.plot(norm_rads, average_sigma_dens_mean, color='blue', label='Revolver centres')
        plt.fill_between(norm_rads, (average_sigma_dens_mean-average_sigma_dens_std),
                         (average_sigma_dens_mean+average_sigma_dens_std), color='none', edgecolor='cyan', hatch="\\\\")
        plt.plot(norm_rads, average_sigma_mean, color='red', label=(r'$\overline{\vert\sigma\vert}$ centres'))
        plt.fill_between(norm_rads, (average_sigma_mean-average_sigma_std),
                         (average_sigma_mean+average_sigma_std), color='none', edgecolor='tomato', hatch="//")
        plt.grid()
        plt.legend(loc='best')
        plt.xlabel(r'$r/R_\mathrm{void}$')
        plt.ylabel(r'$\overline{\vert\sigma\vert} ~ (1/\mathrm{Mpc})$')
        figname=("average_sigma_%i" + ".eps") % void_count
        plt.tight_layout()
        plt.savefig(figname, format = 'eps', metadata = self.metadata)
        plt.close()



    def plot_optical_scalars_vs_weak_lens_cart(self, x_analyse_grid, z_analyse_grid, expansion_dev_map, shear_dev_map, rads, average_deviation_expansions, average_deviation_shears, void_count):

        x_analyse_grid, z_analyse_grid = np.meshgrid(x_analyse_grid, z_analyse_grid, indexing = 'ij', sparse = False)

        fig, ax = plt.subplots(1, 1)
        plt.rcParams.update({'font.size': 14,
                         'font.family': 'serif'})
        plt.ylabel(r'$z ~ (\mathrm{Mpc}/h)$')
        plt.xlabel(r'$x ~ (\mathrm{Mpc}/h)$')
        c = ax.pcolormesh(x_analyse_grid, z_analyse_grid, expansion_dev_map, vmin=np.percentile(expansion_dev_map,10), vmax=np.percentile(expansion_dev_map,90), cmap='cividis')
        cbar=plt.colorbar(c,ax=ax)
        cbar.set_label(r'$\theta / \kappa$', rotation = 270, labelpad=15)
        figname=("theta_vs_kappa_cart_" + "gals_%i" + ".eps") % void_count
        plt.savefig(figname)
        plt.close()

        fig, ax = plt.subplots(1, 1)
        plt.rcParams.update({'font.size': 14,
                         'font.family': 'serif'})
        plt.ylabel(r'$z ~ (\mathrm{Mpc}/h)$')
        plt.xlabel(r'$x ~ (\mathrm{Mpc}/h)$')
        c = ax.pcolormesh(x_analyse_grid, z_analyse_grid, shear_dev_map, vmin=np.percentile(shear_dev_map,10), vmax=np.percentile(shear_dev_map,90), cmap='cividis')
        cbar=plt.colorbar(c,ax=ax)
        cbar.set_label(r'$\sigma / \gamma$', rotation = 270, labelpad=15)
        figname=("sigma_vs_gamma_cart_" + "gals_%i" + ".eps") % void_count
        plt.savefig(figname)
        plt.close()

        plt.plot(rads, average_deviation_shears, 'k-', label='Shear')
        plt.plot(rads, average_deviation_expansions, 'r-', label='Expansion')
        plt.grid()
        plt.legend(loc='best')
        plt.xlabel('r')
        plt.ylabel(r'$Deviation$')
        figname=("average_deviation_%i" + ".eps") % void_count
        plt.savefig(figname, format = 'eps')
        plt.close()


    def plot_void_population(self, void_rad_rev, void_rad_projected, identifier):
        """
        This function creates a histogram visualising the occurance of different void sizes
        in the simulation. Using DM as tracers leads to much smaller voids, this histogram
        should give a feeling for the void sizes.

        Parameter
        ------------
        void_rad_rev: np.array(dtype=float)
        An array of the effective void radii.

        returns
        ------------
        """

        max_void_rad_rev = max(void_rad_rev)
        Nvoids_rev=len(void_rad_rev)
        max_void_rad_projected = max(void_rad_projected)
        Nvoids_projected=len(void_rad_projected)

        bins = np.linspace(0,max(max_void_rad_rev,max_void_rad_projected),20)
        ax=plt.figure()
        plt.rcParams.update({'font.size': 14,
                         'font.family': 'serif'})
        plt.ylabel('$N$')
        plt.xlabel('$R_{\mathrm{eff}}$ (Mpc/$h$)')
        plt.hist(void_rad_rev, bins=bins, linewidth=1,
                 edgecolor ='r', facecolor='none',
                 label='intrinisc voids')
        plt.hist(void_rad_projected, bins=bins, linewidth=1,
                 edgecolor ='b', facecolor='none',
                 label='photometric voids')
        ylims = plt.ylim()
        plt.legend(loc='best')
        plt.ylim((0.0, ylims[1]))
        plt.tight_layout()
        #plt.legend(loc='upper left', numpoints=1, labelspacing=0.1)
        plt.savefig("void_population" + identifier +".eps")
        plt.close()
