#!/usr/bin/env python

import sys

import numpy as np
import matplotlib.pyplot as plt

from scipy.interpolate import LinearNDInterpolator

from figures import figures
from extra_calculations import calculations
from coordinates import coordinate_calc
from lensing_calculations import LensingCalc

if __name__ == '__main__':

    N_stack=int(sys.argv[1])
    print(N_stack)

    box_length=float(sys.argv[2])
    print(box_length)

    from optparse import OptionParser
    import os

    parser = OptionParser()
    parser.add_option(
        '-d',
        '--dir_name',
        dest='DirName',
        default='./',
        help='input directory name (default: ./millennium/)',
        metavar='DIR',
        )

    (opt, args) = parser.parse_args()

    stacking_dir = opt.DirName

    if os.environ.get('OMEGA_M'):
        OmegaM = float(os.environ.get('OMEGA_M'))
    else:
        OmegaM = 0.3
    if os.environ.get('OMEGA_LAMBDA'):
        OmegaLambda = float(os.environ.get('OMEGA_LAMBDA'))
    else:
        OmegaLambda = 0.7

    if os.environ.get('HUBBLE0'):
        Hubble = float(os.environ.get('HUBBLE0'))
    else:
        Hubble = 70.0

    if os.environ.get('SPHERICAL_CALC'):
        if(str(os.environ.get('SPHERICAL_CALC'))=="True"):
            spherical_calc = True
        else:
            spherical_calc = False
    else:
        spherical_calc = False
    if os.environ.get('TEST_WEYL'):
        if(str(os.environ.get('TEST_WEYL'))=="True"):
            do_test_weyl = True
        else:
            do_test_weyl = False
    else:
        do_test_weyl = False
    if os.environ.get('VELOCITY_CENTRE'):
        if(str(os.environ.get('VELOCITY_CENTRE'))=="True"):
            do_velocity_centre = True
        else:
            do_velocity_centre = False
    else:
        do_velocity_centre = False
    if os.environ.get('MASS_DEF'):
        if(str(os.environ.get('MASS_DEF'))=="True"):
            do_mass_def = True
        else:
            do_mass_def = False
    else:
        do_mass_def = False
    if os.environ.get('ISW'):
        if(str(os.environ.get('ISW'))=="True"):
            do_ISW = True
        else:
            do_ISW = False
    else:
        do_ISW = False
    if os.environ.get('DOPPLER_LENSING'):
        if(str(os.environ.get('DOPPLER_LENSING'))=="True"):
            do_doppler_lensing = True
        else:
            do_doppler_lensing = False
    else:
        do_doppler_lensing = False
    if os.environ.get('OPTICAL_SCALARS'):
        if(str(os.environ.get('OPTICAL_SCALARS'))=="True"):
            do_solve_scalars = True
        else:
            do_solve_scalars = False
    else:
        do_solve_scalars = False
    if os.environ.get('STACKING'):
        if(str(os.environ.get('STACKING'))=="YES"):
            stacking_step = True
        else:
            stacking_step = False
    else:
        stacking_step = False

    if os.environ.get('LENSING_DIR'):
        lensing_dir = str(os.environ.get('LENSING_DIR'))
    else:
        lensing_dir = ".build/analysis/lensing-calculations/"

    if os.environ.get('N_PIXELS'):
        N_pixels = int(os.environ.get('N_PIXELS'))
    else:
        N_pixels = 64
    if os.environ.get('MAIN_VOID_REDSHIFT'):
        redshift = float(os.environ.get('MAIN_VOID_REDSHIFT'))
    else:
        redshift = 1.0

    figs=figures()
    calc=calculations()
    coord=coordinate_calc()
    lens_calc=LensingCalc()

    distance = coord.calc_distance(redshift, OmegaM, OmegaLambda, Hubble)

    #gal_redshift = np.load(stacking_dir + "/gal_redshift_1_0.npy")
    #for i in range(2,(N_stack+1)):
    #    gal_redshift = np.append(gal_redshift,np.load(stacking_dir + "/gal_redshift_%i.npy" % i))

    #redshift_mean=np.mean(gal_redshift,axis=0)
    #redshift_std=np.std(gal_redshift,axis=0)
    
    ## Test Weyl ##
    if(do_test_weyl==True):

        x_analyse_grid = np.linspace(0, 1, N_pixels, endpoint=True)
        z_analyse_grid = np.linspace(0, 1, N_pixels, endpoint=True)
        WeylTensor1010 = np.load(stacking_dir + "/WeylTensor1010_1.npy")
        WeylTensor1020 = np.load(stacking_dir + "/WeylTensor1020_1.npy")
        for i in range(2,(N_stack+1)):
            WeylTensor1010 = WeylTensor1010 + np.load(stacking_dir + "/WeylTensor1010_%i.npy" % i)
            WeylTensor1020 = WeylTensor1020 + np.load(stacking_dir + "/WeylTensor1020_%i.npy" % i)

        WeylTensor1010 = WeylTensor1010/N_stack
        WeylTensor1020 = WeylTensor1020/N_stack

        # Set unused detector variables to zero
        signWeyl1010 = np.zeros((N_pixels,N_pixels), dtype=float)
        signWeyl1020 = np.zeros((N_pixels,N_pixels), dtype=float)
        diag1010 = np.zeros((N_pixels,N_pixels), dtype=float)
        diag1010_median = np.zeros((N_pixels,N_pixels), dtype=float)
        poisson_test = np.zeros((N_pixels,N_pixels), dtype=float)
        figs.plot_test_weyl_cart(x_analyse_grid, z_analyse_grid, WeylTensor1010, WeylTensor1020, signWeyl1010, signWeyl1020, diag1010, diag1010_median, poisson_test)

    ## Test Weyl ##

    ## Velcoties ##

    if (do_velocity_centre==True):

        x_analyse_grid = np.linspace(0, 1, N_pixels, endpoint=True)
        z_analyse_grid = np.linspace(0, 1, N_pixels, endpoint=True)

        pixel_radial_vel = np.load(stacking_dir + "/pixel_radial_vel_1.npy")
        pixel_vel_magnitude = np.load(stacking_dir + "/pixel_vel_magnitude_1.npy")
        pixel_vel_los = np.load(stacking_dir + "/pixel_vel_los_1.npy")
        pixel_vel_y = np.load(stacking_dir + "/pixel_vel_y_1.npy")
        for i in range(2,(N_stack+1)):
            pixel_radial_vel = pixel_radial_vel + np.load(stacking_dir + "/pixel_radial_vel_%i.npy" % i)
            pixel_vel_magnitude = pixel_vel_magnitude + np.load(stacking_dir + "/pixel_vel_magnitude_%i.npy" % i)
            pixel_vel_los = pixel_vel_los + np.load(stacking_dir + "/pixel_vel_los_%i.npy" % i)
            pixel_vel_y = pixel_vel_y + np.load(stacking_dir + "/pixel_vel_y_%i.npy" % i)

        pixel_radial_vel = pixel_radial_vel/N_stack
        pixel_vel_magnitude = pixel_vel_magnitude/N_stack
        pixel_vel_los = pixel_vel_los/N_stack
        pixel_vel_y = pixel_vel_y/N_stack

        figs.plot_vels_cart(x_analyse_grid, z_analyse_grid, pixel_radial_vel, pixel_vel_magnitude, pixel_vel_los, pixel_vel_y)

    ## Velcoties ##

    ## Mass deficit ##

    if (do_mass_def==True):

        x_analyse_grid = np.linspace(-(box_length/2), (box_length/2), N_pixels, endpoint=True)
        z_analyse_grid = np.linspace(-(box_length/2), (box_length/2), N_pixels, endpoint=True)

        void_ids = np.load(lensing_dir + "/void_ids_1.npy")
        N_void = len(void_ids)
        eff_void_radius_gals = np.load(lensing_dir + "/eff_void_radius_gals_1.npy")
        mean_void_rad = np.mean(eff_void_radius_gals)

        mass_def = np.zeros((N_pixels, N_pixels))
        dens = np.zeros((N_pixels, N_pixels))
        pixel_pot = np.zeros((N_pixels, N_pixels))
        kappa_grid = np.zeros((N_pixels, N_pixels))
        gamma_grid = np.zeros((N_pixels, N_pixels))
        for i in range(1,(N_stack+1)):
            for j in range(0,N_void):
                mass_def = mass_def + np.load((lensing_dir + "/mass_def_%i_%i.npy") % (i, j))
                dens = dens + np.load((lensing_dir + "/dens_%i_%i.npy") % (i, j))
                pixel_pot = pixel_pot + np.load((lensing_dir + "/pixel_pot_%i_%i.npy") % (i, j))
                kappa_grid = kappa_grid + np.load((lensing_dir + "/kappa_grid_%i_%i.npy") % (i, j))
                gamma_grid = gamma_grid + np.load((lensing_dir + "/gamma_grid_%i_%i.npy") % (i, j))


        mass_def = mass_def/(N_stack*N_void)
        dens = dens/(N_stack*N_void)
        pixel_pot = pixel_pot/(N_stack*N_void)
        kappa_grid = kappa_grid/(N_stack*N_void)
        gamma_grid = gamma_grid/(N_stack*N_void)

        xz_grid_average=[]
        k=0
        mass_def_inter_array=np.zeros((N_pixels*N_pixels))
        for i in range(0,len(x_analyse_grid)):
            for l in range(0,len(z_analyse_grid)):
                xz_grid_average.append([x_analyse_grid[i], z_analyse_grid[l]])
                if(i == (N_pixels-1) and l != (N_pixels-1)):
                    neighbours = [mass_def[i,l], mass_def[0,l], mass_def[i-1,l], mass_def[i,l+1], mass_def[i,l-1]]
                    pixel_median = np.median(neighbours)
                    pixel_mean = np.mean(neighbours)
                if(i != (N_pixels-1) and l == (N_pixels-1)):
                    neighbours = [mass_def[i,l], mass_def[i+1,l], mass_def[i-1,l], mass_def[i,0], mass_def[i,l-1]]
                    pixel_median = np.median(neighbours)
                    pixel_mean = np.mean(neighbours)
                if(i == (N_pixels-1) and l == (N_pixels-1)):
                    neighbours = [mass_def[i,l], mass_def[0,l], mass_def[i-1,l], mass_def[i,0], mass_def[i,l-1]]
                    pixel_median = np.median(neighbours)
                    pixel_mean = np.mean(neighbours)
                if(i != (N_pixels-1) and l != (N_pixels-1)):
                    neighbours = [mass_def[i,l], mass_def[i+1,l], mass_def[i-1,l], mass_def[i,l+1], mass_def[i,l-1]]
                    pixel_median = np.median(neighbours)
                    pixel_mean = np.mean(neighbours)
                if(abs(mass_def[i,l]) > 2*abs(pixel_median)):
                    if(abs(pixel_mean) > 2*abs(pixel_median)):
                        # TODO: probably a faulty integration
                        mass_def[i,l] = pixel_median
                    else:
                        mass_def[i,l] = pixel_median
                mass_def_inter_array[k]=mass_def[i,l]
                k=k+1


        # TODO: Split the weak lensing calculations into an own routine!
        mass_def_inter=LinearNDInterpolator(xz_grid_average, mass_def_inter_array, fill_value=np.mean(mass_def_inter_array))
        mean_mass_def=np.median(mass_def_inter_array)

        rads=np.linspace(0.1, 2*mean_void_rad, 101)
        # TODO: read from somewhere
        mass_dm = 0.857603132724762

        # Overwrting kappa grid here!
        centre, centre_radii, average_mass_def_dens, average_mass_def, average_mass_def_gamma_centre, average_mass_def_combined, average_mass_def_lensing_dens, average_mass_def_lensing, average_mass_def_gamma_centre_lensing, average_mass_def_combined_lensing, kappa_grid, gamma_grid = lens_calc.mass_def_centre(mass_def_inter, x_analyse_grid, z_analyse_grid, rads, mean_mass_def, distance, mass_dm, OmegaM, OmegaLambda, Hubble, mass_def, N_pixels, box_length, mean_void_rad, 0 )

        minima=[centre[0][0][0], centre[0][1][0]]
        #centre, minima = calc.find_minimum(mass_def, x_analyse_grid, z_analyse_grid)
        moved_gal_pos = np.zeros((100,3))
        void_gal_ids = np.array([1,2,3])
        moved_revolver_centres = np.zeros((1,3))
        chosen_void_id = 0

        figs.plot_density_pot_slice(x_analyse_grid, z_analyse_grid, dens, pixel_pot, moved_gal_pos, void_gal_ids, 0 )

        figs.plot_mass_def_cart(x_analyse_grid, z_analyse_grid, mass_def, centre, minima, rads, average_mass_def_dens, average_mass_def, average_mass_def_gamma_centre, centre_radii[1], moved_revolver_centres, chosen_void_id, 0 )

        figs.plot_weak_lensing_cart(x_analyse_grid, z_analyse_grid, kappa_grid, gamma_grid, [centre[1][0][0], centre[1][1][0]], rads, average_mass_def_lensing_dens, average_mass_def_lensing, average_mass_def_gamma_centre_lensing, 0)


    ## Mass deficit ##

    ## ISW ##
    if(do_ISW==True):

        x_analyse_grid = np.linspace(0, 1, N_pixels, endpoint=True)
        z_analyse_grid = np.linspace(0, 1, N_pixels, endpoint=True)
        deviation_to_dens_centre=np.zeros(N_stack)

        ISW = np.load(lensing_dir + "/ISW_1.npy")
        centre, maxima = calc.find_maximum(np.load(lensing_dir + "/ISW_%i.npy" % i), x_analyse_grid, z_analyse_grid)
        deviation_to_dens_centre[0] = calc.calc_centre_deviation([0,0], centre)

        for i in range(2,(N_stack+1)):
            ISW = ISW + np.load(lensing_dir + "/ISW_%i.npy" % i)
            centre, maxima = calc.find_maximum(np.load(lensing_dir + "/ISW_%i.npy" % i), x_analyse_grid, z_analyse_grid)
            deviation_to_dens_centre[i-1] = calc.calc_centre_deviation([0,0], centre)

        ISW = ISW/N_stack

        mean_deviation=np.mean(deviation_to_dens_centre)
        centre, maxima = calc.find_maximum(ISW, x_analyse_grid, z_analyse_grid)

        figs.plot_ISW_cart(x_analyse_grid, z_analyse_grid, ISW, centre, maxima)

    ## ISW ##

    ## Doppler lensing ##
    if(do_doppler_lensing ==True):
        
        x_analyse_grid = np.linspace(0, 1, N_pixels, endpoint=True)
        z_analyse_grid = np.linspace(0, 1, N_pixels, endpoint=True)

        doppler_convergence = np.load(lensing_dir + "/doppler_convergence_1.npy")
        for i in range(2,(N_stack+1)):
            doppler_convergence = doppler_convergence + np.load(lensing_dir + "/doppler_convergence_%i.npy" % i)

        doppler_convergence = doppler_convergence/N_stack

        figs.plot_doppler_cart(x_analyse_grid, z_analyse_grid, doppler_convergence)

    ## Doppler lensing ##

    ## optical scalars ##
    if(do_solve_scalars==True):

        x_analyse_grid = np.linspace(-(box_length/2), (box_length/2), N_pixels, endpoint=True)
        z_analyse_grid = np.linspace(-(box_length/2), (box_length/2), N_pixels, endpoint=True)

        void_ids = np.load(lensing_dir + "/void_ids_1.npy")
        N_void = len(void_ids)
        eff_void_radius_gals = np.load(lensing_dir + "/eff_void_radius_gals_1.npy")
        mean_void_rad = np.mean(eff_void_radius_gals)

        real_theta = np.zeros((N_pixels, N_pixels))
        real_sigma = np.zeros((N_pixels, N_pixels))
        img_sigma = np.zeros((N_pixels, N_pixels))
        mag_sigma = np.zeros((N_pixels, N_pixels))
        for i in range(1,(N_stack+1)):
            for j in range(0,N_void):
                real_theta = real_theta + np.load(lensing_dir + "/real_theta_%i_%i.npy" % (i, j))
                real_sigma = real_sigma + np.load(lensing_dir + "/real_sigma_%i_%i.npy" % (i, j))
                img_sigma = img_sigma + np.load(lensing_dir + "/img_sigma_%i_%i.npy" % (i, j))
                mag_sigma = mag_sigma + np.load(lensing_dir + "/mag_sigma_%i_%i.npy" % (i, j))

        real_theta = real_theta/(N_stack*N_void)
        real_sigma = real_sigma/(N_stack*N_void)
        img_sigma = img_sigma/(N_stack*N_void)
        mag_sigma = mag_sigma/(N_stack*N_void)

        rel_deviation = np.zeros((N_pixels,N_pixels), dtype=float)
        distance_check = np.zeros((N_pixels,N_pixels), dtype=float)

        k=0
        xz_grid_average = []
        expansion_inter_array=np.zeros((N_pixels*N_pixels))
        shear_inter_array=np.zeros((N_pixels*N_pixels))
        for i in range(0,len(x_analyse_grid)):
            for l in range(0,len(z_analyse_grid)):
                xz_grid_average.append([x_analyse_grid[i], z_analyse_grid[l]])
                if(i != (N_pixels-1) and l != (N_pixels-1)):
                    neighbours = [real_theta[i,l], real_theta[i+1,l], real_theta[i-1,l], real_theta[i,l+1], real_theta[i,l-1]]
                    pixel_median = np.median(neighbours)
                    pixel_mean = np.mean(neighbours)
                if(abs(real_theta[i,l]) > 2*abs(pixel_mean)):
                    if(abs(pixel_mean) > 2*abs(pixel_median)):
                        # TODO: probably a faulty integration
                        real_theta[i,l] = pixel_median
                    else:
                        real_theta[i,l] = pixel_mean
                expansion_inter_array[k]=real_theta[i,l]
                shear_inter_array[k]=mag_sigma[i,l]
                k=k+1

        expansion_inter=LinearNDInterpolator(xz_grid_average, expansion_inter_array, fill_value=np.mean(expansion_inter_array))
        shear_inter=LinearNDInterpolator(xz_grid_average, shear_inter_array, fill_value=np.mean(shear_inter_array))
        median_expansion=np.median(real_theta)
        median_shear=np.median(mag_sigma)

        rads=np.linspace(0.1, 2*mean_void_rad, 101)

        centre_exp, radius_exp, average_exp_dens, average_exp_centre=lens_calc.expansion_centre(expansion_inter, x_analyse_grid, z_analyse_grid, rads, median_expansion, box_length, mean_void_rad)
        centre_sigma, radius_sigma, average_sigma_dens, average_sigma_centre=lens_calc.sigma_centre(shear_inter, x_analyse_grid, z_analyse_grid, rads, median_shear, box_length, mean_void_rad)

        moved_revolver_centres=np.zeros((1,3))
        chosen_void_id_exp=0
        chosen_void_id_sig=0

        figs.plot_optical_scalars_cart(x_analyse_grid, z_analyse_grid, rads, real_theta, rel_deviation, real_sigma, img_sigma, distance_check, centre_exp, centre_sigma, average_exp_dens, average_exp_centre, average_sigma_dens, average_sigma_centre, moved_revolver_centres, chosen_void_id_exp, chosen_void_id_sig, 0)

    ## optical scalars ##
