#!/usr/bin/env python

# test_fixed_rng - standalone illustration of reproducible python rng
# (C) 2023 Boud Roukema GPL-3+
#
# This script is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation, either version 3 of the License, or (at your
# option) any later version.
#
# This script is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details. See <http://www.gnu.org/licenses/>.

import numpy as np
from numpy.random import SeedSequence, default_rng, PCG64, Generator

def simulation(fixed_seed):

    # rng_example in the scope of __main__
    x = rng_example.uniform(0.0,20.0,5)
    print("rng_example=",rng_example)
    print("x=",x)

if __name__ == '__main__':

    # fixed seed and fixed generator PCG64;
    # [1] https://www.pcg-random.org
    # [2] https://numpy.org/doc/stable/reference/random/bit_generators/pcg64.html#numpy.random.PCG64

    seeds = SeedSequence(8623)
    #seed = seeds.spawn(3) # this is useful for seeds for parallel runs


    print("With a fixed seed:")
    rng_example = Generator(PCG64(seeds))
    simulation(True)

    rng_example = Generator(PCG64(seeds)) # restart
    simulation(True)

    rng_example = Generator(PCG64(seeds)) # restart
    simulation(True)

    print("\n  ... same seed, but do not restart generator:")
    simulation(True)

    rng_example = default_rng() # restart
    print("\nWith a non-fixed seed:")
    simulation(False)
    simulation(False)
    simulation(False)
