/*
test_gsl_lensing - test the solution of the differential equations of the void lensing project
Copyright (C) 2022 Marius Peper, Boud Roukema

The purpose of this file is to solve the differential equations of the lensing project.

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

*/

#include <stdio.h>
#include <math.h>

/* prototypes for functions in lensing_gsl.c */
double R44_model(double r, double shift_x, double shift_y, double amp);

double C1010_model(double r, double shift_x, double shift_y, double amp);

double *optical_scalars(double *r, double *R44, double *C1010, double *C1020, double t, double t1);


int main (void)
{
  double amp_R = 0.3e-6;
  double shift_x_R = 50;
  double shift_y_R = 1.e-4;
  double amp_C = 0.5e-7;
  double shift_x_C = 90;
  double shift_y_C = 2.e-4;


  int i;
#define N_AFFINE 150
  double R44[N_AFFINE];
  double C1010[N_AFFINE];
  double C1020[N_AFFINE];
  double r[N_AFFINE];
  double *test;

  int pass=0;
  double check_value=0.0000;
  double check_tol=1e-6;
  
  for(i=0;i<N_AFFINE;i++)
    {
      r[i] = (double) i;
      R44[i] = R44_model(r[i], shift_x_R, shift_y_R, amp_R);
      C1010[i] = C1010_model(r[i], shift_x_C, shift_y_C, amp_C);
      C1020[i] = 1.3*C1010[i];
    }
  printf("solve diff\n");
  test=optical_scalars(r, R44, C1010, C1020, 4, 99);
  printf("*test=%f\n",*test);
  //optical_scalars();

  if(fabs(*test-check_value) < check_tol){
    pass = 0;
  }else{
    pass = 1;
  };
  printf("pass=%d\n",pass);

  return pass;
}
